@@ -1,5 +1,6 @@
 /* -*- mode:C++; tab-width:8; c-basic-offset:8; indent-tabs-mode:true -*- */
 
+
 /*
  * written by J. Marcin Gorycki <marcin.gorycki@intel.com>
  *
@@ -32,6 +33,7 @@
 #include <games-gconf.h>
 #include <games-frame.h>
 #include <games-controls.h>
+#include <games-stock.h>
 
 #include <libgnomevfs/gnome-vfs.h>
 
@@ -110,17 +112,58 @@ Tetris::Tetris(int cmdlLevel):
 	fastFall(false),
         dropBlock(false)
 {
-	double x1, y1, x2, y2;
-	double width;
-	double pts;
-	gchar * outdir;
-	GtkTargetEntry targets[] = {{"text/uri-list", 0, URI_LIST}, 
-				    {"property/bgimage", 0, URI_LIST},
-				    {"text/plain", 0, TEXT_PLAIN},
-				    {"STRING", 0, TEXT_PLAIN},
-				    {"application/x-color", 0, COLOUR},
-				    {"x-special/gnome-reset-background", 0, RESET}};
+	GtkUIManager *ui_manager;
+	GtkAccelGroup *accel_group;
+	GtkActionGroup *action_group;
+	GtkWidget *vbox;
+	GtkWidget *menubar;
 
+	gchar *outdir;
+	const GtkTargetEntry targets[] = {{"text/uri-list", 0, URI_LIST}, 
+					  {"property/bgimage", 0, URI_LIST},
+					  {"text/plain", 0, TEXT_PLAIN},
+					  {"STRING", 0, TEXT_PLAIN},
+					  {"application/x-color", 0, COLOUR},
+					  {"x-special/gnome-reset-background", 0, RESET}};
+
+	const GtkActionEntry actions[] = {
+	{ "GameMenu", NULL, N_("_Game") },
+	{ "SettingsMenu", NULL, N_("_Settings") },
+	{ "HelpMenu", NULL, N_("_Help") },
+	{ "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, G_CALLBACK (gameNew) },
+	{ "Pause", GAMES_STOCK_PAUSE_GAME, NULL, NULL, NULL, G_CALLBACK (gamePause) },
+	{ "Resume", GAMES_STOCK_RESUME_GAME, NULL, NULL, NULL, G_CALLBACK (gamePause) },
+	{ "Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL, G_CALLBACK (gameTopTen) },
+	{ "EndGame", GAMES_STOCK_END_GAME, NULL, NULL, NULL, G_CALLBACK (gameEnd) },
+	{ "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (gameQuit) },
+	{ "Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL, G_CALLBACK (gameProperties) },
+	{ "Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL, G_CALLBACK (gameHelp) },
+	{ "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (gameAbout) }
+	};
+
+	const char ui_description[] =
+	"<ui>"
+	"  <menubar name='MainMenu'>"
+	"    <menu action='GameMenu'>"
+	"      <menuitem action='NewGame'/>"
+	"      <menuitem action='Pause'/>"
+	"      <menuitem action='Resume'/>"
+	"      <separator/>"
+	"      <menuitem action='Scores'/>"
+	"      <menuitem action='EndGame'/>"
+	"      <separator/>"
+	"      <menuitem action='Quit'/>"
+	"    </menu>"
+	"    <menu action='SettingsMenu'>"
+	"      <menuitem action='Preferences'/>"
+	"    </menu>"
+	"    <menu action='HelpMenu'>"
+	"      <menuitem action='Contents'/>"
+	"      <menuitem action='About'/>"
+	"    </menu>"
+	"  </menubar>"
+	"</ui>";
+	
 	if (!sound)
 		sound = new Sound();
 
@@ -147,43 +190,6 @@ Tetris::Tetris(int cmdlLevel):
 	g_signal_connect (G_OBJECT (w), "focus_out_event",
 			  G_CALLBACK (focusOut), this);
 
-	static GnomeUIInfo game_menu[] = 
-	{
-		GNOMEUIINFO_MENU_NEW_GAME_ITEM(gameNew, this),
-		GNOMEUIINFO_MENU_PAUSE_GAME_ITEM(gamePause, this),
-		GNOMEUIINFO_SEPARATOR,
-		GNOMEUIINFO_MENU_SCORES_ITEM(gameTopTen, this),
-		GNOMEUIINFO_MENU_END_GAME_ITEM(gameEnd, this),
-		GNOMEUIINFO_SEPARATOR,
-		GNOMEUIINFO_MENU_QUIT_ITEM(gameQuit, this),
-		GNOMEUIINFO_END
-	};
-
-	gameMenuPtr = game_menu;
-	
-	static GnomeUIInfo settings_menu[] = 
-	{
-		GNOMEUIINFO_MENU_PREFERENCES_ITEM(gameProperties, this),
-		GNOMEUIINFO_END
-	};
-
-	gameSettingsPtr = settings_menu;
-	
-	GnomeUIInfo help_menu[] = 
-	{
-		GNOMEUIINFO_HELP((gpointer)"gnometris"),
-		GNOMEUIINFO_MENU_ABOUT_ITEM(gameAbout, this),
-		GNOMEUIINFO_END
-	};
-
-	GnomeUIInfo mainmenu[] = 
-	{
-		GNOMEUIINFO_MENU_GAME_TREE(game_menu),
-		GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu),
-		GNOMEUIINFO_MENU_HELP_TREE(help_menu),
-		GNOMEUIINFO_END
-	};
-
 	line_fill_height = 0;
 	line_fill_prob = 5;
 
@@ -199,12 +205,36 @@ Tetris::Tetris(int cmdlLevel):
 
 	initOptions ();
 
-	gnome_app_create_menus(GNOME_APP(w), mainmenu);
+	/* prepare menus */
+	games_stock_init ();
+	action_group = gtk_action_group_new ("MenuActions");
+	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (action_group, actions, G_N_ELEMENTS (actions), this);
+	ui_manager = gtk_ui_manager_new ();
+	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+	gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
+	accel_group = gtk_ui_manager_get_accel_group (ui_manager);
+	gtk_window_add_accel_group (GTK_WINDOW (w), accel_group);
+
+	new_game_action = gtk_action_group_get_action (action_group, "NewGame");
+	pause_action = gtk_action_group_get_action (action_group, "Pause");
+	resume_action = gtk_action_group_get_action (action_group, "Resume");
+	scores_action = gtk_action_group_get_action (action_group, "Scores");
+	end_game_action = gtk_action_group_get_action (action_group, "EndGame");
+	preferences_action = gtk_action_group_get_action (action_group, "Preferences");
+
+	games_stock_set_pause_actions (pause_action, resume_action);
+
+	menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
 
 	GtkWidget * hb = gtk_hbox_new(FALSE, 0);
-	gnome_app_set_contents(GNOME_APP(w), hb);
 
-	field = new Field(/*ops*/);
+	vbox = gtk_vbox_new (FALSE, 0);
+	gnome_app_set_contents (GNOME_APP (w), vbox);
+	gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hb, TRUE, TRUE, 0);
+
+	field = new Field();
 	ops = new BlockOps(field);
 
 	gtk_widget_set_events(w, gtk_widget_get_events(w) | 
@@ -214,7 +244,7 @@ Tetris::Tetris(int cmdlLevel):
 	gtk_container_set_border_width(GTK_CONTAINER(vb1), 10);
 	gtk_box_pack_start_defaults(GTK_BOX(vb1), field->getWidget());
 	gtk_box_pack_start(GTK_BOX(hb), vb1, 0, 0, 0);
-	field->show();
+	gtk_widget_show(field->getWidget());
 	setupPixmap();
 
 	g_signal_connect (w, "key_press_event", G_CALLBACK (keyPressHandler), this);
@@ -225,7 +255,7 @@ Tetris::Tetris(int cmdlLevel):
 	gtk_box_pack_end(GTK_BOX(hb), vb2, 0, 0, 0);
 	
 	preview = new Preview();
-	
+
 	gtk_box_pack_start(GTK_BOX(vb2), preview->getWidget(), FALSE, FALSE, 0);
 	
 	scoreFrame = new ScoreFrame(cmdlineLevel);
@@ -233,7 +263,7 @@ Tetris::Tetris(int cmdlLevel):
 	gtk_box_pack_end(GTK_BOX(vb2), scoreFrame->getWidget(), TRUE, FALSE, 0);
 
 	setOptions ();
-        setupScoreState ();
+	setupScoreState ();
 
 	themeList = NULL;
 	
@@ -243,57 +273,9 @@ Tetris::Tetris(int cmdlLevel):
 	scoreFrame->show();
 	gtk_widget_show(w);
 
-	gtk_widget_set_sensitive(gameMenuPtr[1].widget, FALSE);
-	gtk_widget_set_sensitive(gameMenuPtr[4].widget, FALSE);
-	gtk_widget_set_sensitive(gameSettingsPtr[0].widget, TRUE);
-
-        pauseMessage = gnome_canvas_item_new (gnome_canvas_root(GNOME_CANVAS(field->getWidget())),
-                                              gnome_canvas_text_get_type(),
-                                              "fill_color",
-                                              "white",
-                                              "x", COLUMNS*BLOCK_SIZE/2.0,
-                                              "y", LINES*BLOCK_SIZE/2.0,
-                                              "text", _("Paused"),
-                                              "size_points", 36.0,
-                                              NULL
-                                              );
-
-	/* Since gnome_canvas doesn't support setting the size of text in
-	 * pixels (read the source where the "size" parameter gets set)
-	 * and pango isn't forthcoming about how it scales things (see
-	 * http://mail.gnome.org/archives/gtk-i18n-list/2003-August/msg00001.html
-	 * and bug #119081). We guess at the size, see what size it is rendered
-	 * to and then adjust the point size to fit. 36.0 points is pretty
-	 * close for 96 dpi . */
-
-	gnome_canvas_item_get_bounds (pauseMessage, &x1, &y1, &x2, &y2);
-	width = x2 - x1;
-	/* 0.8 is the fraction of the screen we want to use and 36.0 is
-	 * the guess we use previously for the point size. */
-	pts = 0.8*36.0*COLUMNS*BLOCK_SIZE/width;
-	gnome_canvas_item_set (pauseMessage, "size_points", pts, 0);
-
-        gnome_canvas_item_hide (pauseMessage);
-
-        gameoverMessage = gnome_canvas_item_new (gnome_canvas_root(GNOME_CANVAS(field->getWidget())),
-                                                 gnome_canvas_text_get_type(),
-                                                 "fill_color",
-                                                 "white",
-                                                 "x", COLUMNS*BLOCK_SIZE/2.0,
-                                                 "y", LINES*BLOCK_SIZE/2.0,
-                                                 "text", _("Game Over"),
-                                                 "size_points", 36.0,
-                                                 NULL
-                                                 );
-
-	gnome_canvas_item_get_bounds (gameoverMessage, &x1, &y1, &x2, &y2);
-	width = x2 - x1;
-	/* 0.9 is the fraction of the screen we want to use and 36.0 is
-	 * the guess we use previously for the point size. */
-	pts = 0.9*36.0*COLUMNS*BLOCK_SIZE/width;
-	gnome_canvas_item_set (gameoverMessage, "size_points", pts, 0);
-
-        gnome_canvas_item_hide (gameoverMessage);
+	gtk_action_set_sensitive(pause_action, FALSE);
+	gtk_action_set_sensitive(end_game_action, FALSE);
+	gtk_action_set_sensitive(preferences_action, TRUE);
 
 	high_scores = new HighScores ();
 }
@@ -328,12 +310,12 @@ Tetris::setupScoreState ()
 
 	top = gnome_score_get_notable ("gnometris", NULL, &names, &scores, &scoretimes);
 	if (top > 0) {
-		gtk_widget_set_sensitive (gameMenuPtr[3].widget, TRUE);
+		gtk_action_set_sensitive (scores_action, TRUE);
 		g_strfreev (names);
 		g_free (scores);
 		g_free (scoretimes);
 	} else {
-		gtk_widget_set_sensitive (gameMenuPtr[3].widget, FALSE);
+		gtk_action_set_sensitive (scores_action, FALSE);
 	}
 }
 
@@ -341,10 +323,12 @@ void
 Tetris::setupdialogDestroy(GtkWidget *widget, void *d)
 {
 	Tetris *t = (Tetris*) d;
-	if (t->setupdialog)
+	if (t->setupdialog) {
+		delete t->theme_preview;
 		gtk_widget_destroy(t->setupdialog);
+	}
 	t->setupdialog = 0;
-	gtk_widget_set_sensitive(t->gameMenuPtr[0].widget, TRUE);
+	gtk_action_set_sensitive(t->new_game_action, TRUE);
 }
 
 void
@@ -560,6 +544,8 @@ Tetris::initOptions ()
 		sound->turnOff ();
 
 	do_preview = gconfGetBoolean (gconf_client, KEY_DO_PREVIEW, TRUE);
+	if (preview)
+		preview->enable(do_preview);
 
 	random_block_colors = gconfGetBoolean (gconf_client, KEY_RANDOM_BLOCK_COLORS, TRUE);
 
@@ -602,6 +588,9 @@ Tetris::setOptions ()
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (do_preview_toggle), do_preview);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (random_block_colors_toggle), random_block_colors);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rotate_counter_clock_wise_toggle), rotateCounterClockWise);
+
+		if (theme_preview)
+			gtk_widget_queue_draw(theme_preview->getWidget());
 	}
 
 	scoreFrame->setLevel (startingLevel);
@@ -741,7 +730,7 @@ Tetris::fillMenu(GtkWidget *menu, char *pixname, char *dirname,
 }
 
 int 
-Tetris::gameProperties(GtkWidget *widget, void *d)
+Tetris::gameProperties(GtkAction *action, void *d)
 {
 	GtkWidget *notebook;
 	GtkWidget *vbox;
@@ -751,7 +740,7 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	GtkWidget *fvbox;
 	GtkObject *adj;
 	GtkWidget *controls_list;
-        
+
 	Tetris *t = (Tetris*) d;
 	
 	if (t->setupdialog) {
@@ -862,7 +851,6 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 			  G_CALLBACK (setSound), d);
 	gtk_box_pack_start (GTK_BOX (fvbox), t->sound_toggle, 0, 0, 0);
 	
-
 	/* preview next block */
 	t->do_preview_toggle =
 		gtk_check_button_new_with_mnemonic (_("_Preview next block"));
@@ -904,25 +892,6 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
 
-	/* Block pixmap */
-	label = gtk_label_new_with_mnemonic (_("_Block image:"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
-	                  (GtkAttachOptions) 0, 
-			  (GtkAttachOptions) 0,
-			  0, 0);
-
-	GtkWidget *omenu = gtk_combo_box_new_text ();
-	g_object_set_data (G_OBJECT (omenu), TETRIS_OBJECT, t);	
-	t->fillMenu (omenu, t->blockPixmap, "gnometris", &(t->themeList));
-	g_signal_connect (omenu, "changed", G_CALLBACK (setSelection), NULL);
-	gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), omenu);
-
-	gtk_container_add (GTK_CONTAINER (frame), table);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, 
-			    FALSE, FALSE, 0);
-
 	/* controls page */
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
@@ -930,6 +899,8 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, label);
 
 	frame = games_frame_new (_("Keyboard Controls"));
+	gtk_container_add (GTK_CONTAINER (vbox), frame);
+
 	fvbox = gtk_vbox_new (FALSE, 6);
 	gtk_container_add (GTK_CONTAINER (frame), fvbox);
 
@@ -945,10 +916,32 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 
 	gtk_box_pack_start (GTK_BOX (fvbox), controls_list, TRUE, TRUE, 0);
 
+	/* theme page */
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
+	label = gtk_label_new (_("Theme"));
+	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, label);
+
+	frame = games_frame_new (_("Block image:"));
 	gtk_container_add (GTK_CONTAINER (vbox), frame);
-	
+
+	fvbox = gtk_vbox_new (FALSE, 6);
+	gtk_container_add (GTK_CONTAINER (frame), fvbox);
+
+	GtkWidget *omenu = gtk_combo_box_new_text ();
+	g_object_set_data (G_OBJECT (omenu), TETRIS_OBJECT, t);	
+	t->fillMenu (omenu, t->blockPixmap, "gnometris", &(t->themeList));
+	g_signal_connect (omenu, "changed", G_CALLBACK (setSelection), NULL);
+	gtk_box_pack_start (GTK_BOX (fvbox), omenu, FALSE, FALSE, 0);
+
+	t->theme_preview = new Preview();
+	gtk_box_pack_start(GTK_BOX(fvbox), t->theme_preview->getWidget(), TRUE, TRUE, 0);
+
+	t->theme_preview->previewBlock(4, 0, 0);
+
 	gtk_widget_show_all (t->setupdialog);
-	gtk_widget_set_sensitive(t->gameMenuPtr[0].widget, FALSE);
+	gtk_action_set_sensitive(t->new_game_action, FALSE);
+
 	return TRUE;
 }
 
@@ -961,7 +954,7 @@ Tetris::focusOut(GtkWidget *widget, GdkEvent *e, Tetris *t)
 }
 
 int
-Tetris::gamePause(GtkWidget *widget, void *d)
+Tetris::gamePause(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
 	t->togglePause();
@@ -969,7 +962,7 @@ Tetris::gamePause(GtkWidget *widget, void *d)
 }
 
 int
-Tetris::gameEnd(GtkWidget *widget, void *d)
+Tetris::gameEnd(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
 
@@ -981,7 +974,7 @@ Tetris::gameEnd(GtkWidget *widget, void *d)
 }
 
 int
-Tetris::gameQuit(GtkWidget *widget, void *d)
+Tetris::gameQuit(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
 
@@ -1015,7 +1008,10 @@ Tetris::manageFallen()
 	sound->playSound (SOUND_LAND);
 
 	int levelBefore = scoreFrame->getLevel();
-	ops->checkFullLines(scoreFrame);
+
+	int fullLines = ops->checkFullLines();
+	if (fullLines > 0)
+		scoreFrame->incLines(fullLines);
 	int levelAfter = scoreFrame->getLevel();
 	if (levelAfter != levelBefore) 
 		sound->playSound (SOUND_GNOMETRIS);
@@ -1342,11 +1338,16 @@ void
 Tetris::togglePause()
 {
 	paused = !paused;
-        if (paused) {
-                gnome_canvas_item_show (pauseMessage);
-                gnome_canvas_item_raise_to_top (pauseMessage);
-        } else
-                gnome_canvas_item_hide (pauseMessage);
+
+	if (paused)
+		field->showPauseMessage();
+	else
+		field->hidePauseMessage();
+
+	gtk_action_set_sensitive (pause_action, !paused);
+	gtk_action_set_sensitive (resume_action, paused);
+	gtk_action_set_visible (pause_action, !paused);
+	gtk_action_set_visible (resume_action, paused);
 }
 
 void
@@ -1355,6 +1356,7 @@ Tetris::generate()
 	if (ops->generateFallingBlock())
 	{
 		ops->putBlockInField(false);
+		preview->previewBlock(blocknr_next, rot_next, color_next);
 		gtk_widget_queue_draw(preview->getWidget());
 		onePause = true;
 	}
@@ -1371,16 +1373,18 @@ Tetris::generate()
 void
 Tetris::endOfGame()
 {
-	gtk_widget_set_sensitive(gameMenuPtr[1].widget, FALSE);
-	gtk_widget_set_sensitive(gameMenuPtr[4].widget, FALSE);
-	gtk_widget_set_sensitive(gameSettingsPtr[0].widget, TRUE);
+	if (paused) togglePause();
+	gtk_action_set_sensitive (pause_action, FALSE);
+	gtk_action_set_sensitive (end_game_action, FALSE);
+	gtk_action_set_sensitive (preferences_action, TRUE);
+
 	color_next = -1;
 	blocknr_next = -1;
 	rot_next = -1;
+	preview->previewBlock(-1, -1, -1);
 	gtk_widget_queue_draw(preview->getWidget());
-        gnome_canvas_item_hide (pauseMessage);
-        gnome_canvas_item_show (gameoverMessage);
-        gnome_canvas_item_raise_to_top (gameoverMessage);
+	field->hidePauseMessage();
+	field->showGameOverMessage();
 	sound->playSound (SOUND_GAMEOVER);
 	inPlay = false;
 
@@ -1392,7 +1396,7 @@ Tetris::endOfGame()
 }
 
 int
-Tetris::gameNew(GtkWidget *widget, void *d)
+Tetris::gameNew(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
 
@@ -1424,22 +1428,33 @@ Tetris::gameNew(GtkWidget *widget, void *d)
 	
 	t->ops->generateFallingBlock();
 	gtk_widget_queue_draw(t->field->getWidget());
+	t->preview->previewBlock(blocknr_next, rot_next, color_next);
 	gtk_widget_queue_draw(t->preview->getWidget());
 
-	gtk_widget_set_sensitive(t->gameMenuPtr[1].widget, TRUE);
-	gtk_widget_set_sensitive(t->gameMenuPtr[4].widget, TRUE);
-	gtk_widget_set_sensitive(t->gameSettingsPtr[0].widget, FALSE);
+	gtk_action_set_sensitive(t->pause_action, TRUE);
+	gtk_action_set_sensitive(t->end_game_action, TRUE);
+	gtk_action_set_sensitive(t->preferences_action, FALSE);
+
+	t->field->hidePauseMessage();
+	t->field->hideGameOverMessage();
 
-        gnome_canvas_item_hide (t->pauseMessage);
-        gnome_canvas_item_hide (t->gameoverMessage);
-        
 	sound->playSound (SOUND_GNOMETRIS);
 
 	return TRUE;
 }
 
 int
-Tetris::gameAbout(GtkWidget *widget, void *d)
+Tetris::gameHelp(GtkAction *action, void *d)
+{
+	Tetris *t = (Tetris*) d;
+
+	gnome_help_display ("gnometris.xml", NULL, NULL);
+
+	return TRUE;
+}
+
+int
+Tetris::gameAbout(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
 
@@ -1449,7 +1464,7 @@ Tetris::gameAbout(GtkWidget *widget, void *d)
 			       "name", _("Gnometris"),
 			       "version", VERSION,
 			       "comments", _("Written for my wife, Matylda\n"
-					     "Send comments and bug reports to: "
+					     "Send comments and bug reports to: \n"
 					     "janusz.gorycki@intel.com"),
 				"copyright", "Copyright \xc2\xa9 2000 J. Marcin Gorycki",
 				"authors", authors,				
@@ -1460,7 +1475,7 @@ Tetris::gameAbout(GtkWidget *widget, void *d)
 }
 
 int
-Tetris::gameTopTen(GtkWidget *widget, void *d)
+Tetris::gameTopTen(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
 	t->high_scores->show(0);
